#include "StdAfx.h"
#include "TipDocument.h"
#include "XMLUtil.h"
#include <atlpath.h>
#include "HttpClient/httpclient.h"
#include "Binding.h"
#include "ScriptletContainerFactory.h"
#include "Application.h"
#include "CustomProgressBar.h"


namespace tipcommon
{
TipDocument::TipDocument(void):m_DocTemp(NULL),m_rootDataContext(NULL),m_valueValidater(NULL)
{
};

TipDocument::~TipDocument()
{
	if (m_rootDataContext)
	{
		delete m_rootDataContext;
		m_rootDataContext = NULL;
	}
}


BOOL IsWow64()
{	
	typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);

	LPFN_ISWOW64PROCESS fnIsWow64Process = NULL;

	BOOL bIsWow64 = FALSE;

	//IsWow64Process is not available on all supported versions of Windows.
	//Use GetModuleHandle to get a handle to the DLL that contains the function
	//and GetProcAddress to get a pointer to the function if available.

	fnIsWow64Process = (LPFN_ISWOW64PROCESS) GetProcAddress( GetModuleHandle(TEXT("kernel32")), "IsWow64Process");

	if(NULL != fnIsWow64Process)
	{
		if (!fnIsWow64Process(GetCurrentProcess(),&bIsWow64))
		{
			//handle error
		}
	}

	return bIsWow64;
}


BOOL TipDocument::Open()
{
	m_rootDataContext = new SimpleMapDataContext;

	return TRUE;
}

BOOL TipDocument::Close()
{
	if (!m_ServiceList.empty())
	{
		std::vector<TipServiceData*>::iterator it = m_ServiceList.begin();
		for (; it != m_ServiceList.end(); it++)
		{
			(*it)->UnregisterAgentDll();
			delete *it;
		}
		m_ServiceList.clear();
	}
	return TRUE;
}
int TipDocument::GetCountOfServiceData(LPCTSTR name) 
{
	IDataContext* dataContext = this->GetChild(name);
	if (dataContext != NULL)
	{
		return dataContext->GetCount();
	}
	return 0;
}
TipServiceData* TipDocument::GetServiceData(LPCTSTR name, int index)
{
	IDataContext* installlistDataContext = this->GetChild(name);
	if (installlistDataContext != NULL)
	{
		IDataContext* serviceDataContext = installlistDataContext->GetChild(index);
		if (serviceDataContext == NULL) return NULL;

		CComVariant var = serviceDataContext->GetProperty(_T("helperobject"));
		return reinterpret_cast<TipServiceData*>(UIntToPtr(var.uintVal));
	}
	return NULL;
}
TipServiceData* TipDocument::SearchService(const TCHAR* guid)
{	
	// GUID  ˻
	TipServiceData* service = NULL;
	std::vector<TipServiceData*>::iterator it = m_ServiceList.begin();

	for (; it != m_ServiceList.end(); it++)
	{
		service = *it;

		if (_tcscmp(guid, service->GetID()) == 0)
			return (*it);
	}
	return NULL;
}

void TipDocument::AddDataContext(LPCTSTR name)
{
	ATLASSERT(m_rootDataContext);
	m_rootDataContext->SetProperty(name, CComVariant(PtrToUint(new SimpleMapDataContext())));
}

IDataContext* TipDocument::GetChild(LPCTSTR name)
{
	ATLASSERT(m_rootDataContext);
	IDataContext* childDataContext = m_rootDataContext->GetChild(name);
	return childDataContext;
}

IDataContext* TipDocument::GetRoot()
{
	ATLASSERT(m_rootDataContext);
	return m_rootDataContext;
}


BOOL TipDocument::LoadXmlDoc()
{
	m_rootDataContext = new SimpleMapDataContext;

	TiXmlElement* element =  m_DocTemp->FirstChildElement();
	XMLDataProvider xmlProvider(m_rootDataContext);
	xmlProvider.SetValidater(m_valueValidater);
	xmlProvider.Load(element);
	BOOL bXmlDataValid = TRUE;
	if (m_valueValidater)
	{
		bXmlDataValid = m_valueValidater->IsXMLDataValid();
		delete m_valueValidater;
		m_valueValidater = NULL;
	}
		
	REGISTER_DATACONTEXT(_T("Document"), m_rootDataContext);
	return bXmlDataValid;
}

CComVariant TipDocument::GetProperty(LPCTSTR key)
{
	ATLASSERT(m_rootDataContext);
	return m_rootDataContext->GetProperty(key);	
}

void TipDocument::SetProperty(LPCTSTR key, CComVariant data)
{
	ATLASSERT(m_rootDataContext);
	m_rootDataContext->SetProperty(key, data);
}
void TipDocument::OpenUrlWithDefaultBrowser(LPCTSTR url)
{
	if (url == NULL)
		return;
	// Find Default Browser

	::ShellExecute( NULL, _T("Open"), _T("IEXPLORE.EXE"), url, NULL, SW_SHOW );
//	ShellExecuteEx(&sei);
}

void TipDocument::ClearServiceData()
{
	m_ServiceList.clear();
}

void TipDocument::AddServiceData( TipServiceData* service )
{
	m_ServiceList.push_back(service);
}


void TipDocument::SetDefaultListIndex( int index )
{
	if ( -1 == index )
	{
		index = 0;
	}
	SetProperty(L"default_select", index);
}

int TipDocument::GetDefaultListIndex()
{
	CComVariant var = GetProperty( L"default_select");
	if( var.vt == VT_I4 )
	{
		return var.intVal;
	}
	return 0;
}

static LPCWSTR kPartner = L"partner";
static LPCWSTR kBigIntroIamgeUrl = L"(bigintroimage.url)";
//////////////////////////////////////////////////////////////////////////
//TipServiceData
//////////////////////////////////////////////////////////////////////////
TipServiceData::TipServiceData(TipDocument* document, IDataContext* datacontext)
:m_document(document), m_datacontext(datacontext), m_agentDllWrapper(NULL) 
{
	ATLASSERT(m_document && m_datacontext);
	m_datacontext->SetProperty(_T("helperobject"), CComVariant(PtrToUint(this)));
	m_datacontext->SetProperty(_T("installstatus"), InstallStatus::INIT);
	m_datacontext->SetProperty(_T("progress_position"), 0);
	m_datacontext->SetProperty(_T("visible"), CComVariant(TRUE));

	
	//		
	//if( GetID().CompareNoCase(_T("44CC52A5-1C07-42a5-9612-D7B79543F62B")) == 0 )
	//{ // ĳ
	//	m_datacontext->SetProperty(_T("name_margin"), CComVariant(_T("18 10 0 0")));
	//	m_datacontext->SetProperty(_T("state_init_margin"), CComVariant(_T("0 15 10 0")));
	//	m_datacontext->SetProperty(_T("progress_bar_margin"), CComVariant("0 18 0 0"));
	//	m_datacontext->SetProperty(_T("cancel_button_margin"), CComVariant("0 17 18 0"));
	//}
	//else
	//{
	//	m_datacontext->SetProperty(_T("name_margin"), CComVariant(_T("18 15 0 0")));
	//	m_datacontext->SetProperty(_T("state_init_margin"), CComVariant("0 15 10 0"));
	//	m_datacontext->SetProperty(_T("progress_bar_margin"), CComVariant("0 18 0 0"));
	//	m_datacontext->SetProperty(_T("cancel_button_margin"), CComVariant("0 17 18 0"));
	//}
}
TipServiceData::~TipServiceData()
{ 
	UnregisterAgentDll(); 
}
void TipServiceData::RegisterAgentDll()
{
	if (m_agentDllWrapper)
	{
		AgentDLLManager::Unregister(m_agentDllWrapper);
	}
	m_agentDllWrapper = AgentDLLManager::Register(this->GetID(), this->GetAgentDataURL());
}

AgentDLLWrapper* TipServiceData::GetAgentDll()
{
	if (m_agentDllWrapper == NULL)
	{
		RegisterAgentDll();
	}
	return m_agentDllWrapper;
}

void TipServiceData::UnregisterAgentDll()
{
	if (m_agentDllWrapper)
	{
		AgentDLLManager::Unregister(m_agentDllWrapper);
	}
	m_agentDllWrapper = NULL;
}

void TipServiceData::SetInstallInfo(InstallInfo::type info)
{
}

CString TipServiceData::GetBigIntroImageUrl()
{
	ATLASSERT(m_datacontext);
	CComVariant var = m_datacontext->GetProperty( kBigIntroIamgeUrl );
	if( var.vt != VT_BSTR)
	{
		ATLTRACE(L"Bingintroimage is not vt_bstr");
		return L"";
	}
	return CString(var.bstrVal);
}

BOOL TipServiceData::IsPartner()
{
	ATLASSERT(m_datacontext);
	CComVariant var = m_datacontext->GetProperty( kPartner);
	return ScriptUtil::Varient2BOOL(&var);
}

BOOL TipServiceData::IsNewIcon()
{
	ATLASSERT(m_datacontext);
	CComVariant var = m_datacontext->GetProperty( L"(newicon.show)");
	return ScriptUtil::Varient2BOOL(&var);
}

CString TipServiceData::GetMainfuncURL()
{
	ATLASSERT(m_datacontext);
	CComVariant var = m_datacontext->GetProperty( L"(mainfunction.url)");
	if( var.vt != VT_BSTR )
	{
		return GetIntroduceURL();
	}

	CString url = CString(var.bstrVal);
	if( url.GetLength() == 0 )
	{
		return GetIntroduceURL();
	}
	return url;
}

BOOL TipXMLValidater::IsValidURL(LPCTSTR data)
{
	TCHAR regExp[] = _T("^(http:\\/\\/).*$");
	return IsValidWithRegExp(data, regExp);
}
BOOL TipXMLValidater::IsValidGUID(LPCTSTR data)
{
	TCHAR regExp[] = _T("^\\{?((\\h\\h\\h\\h\\h\\h\\h\\h)-(\\h\\h\\h\\h)-(\\h\\h\\h\\h)-(\\h\\h\\h\\h)-(\\h\\h\\h\\h\\h\\h\\h\\h\\h\\h\\h\\h))\\}?$");
	return IsValidWithRegExp(data, regExp);
}
BOOL TipXMLValidater::IsValidNumber(LPCTSTR data)
{
	TCHAR regExp[] = _T("^(\\d+\\.?)(\\d+)?$");
	return IsValidWithRegExp(data, regExp);
}
BOOL TipXMLValidater::IsValidBoolean(LPCTSTR data)
{
	TCHAR regExp[] = _T("^(True)?(False)?$");
	return IsValidWithRegExp(data, regExp);
}
BOOL TipXMLValidater::IsValidWithRegExp(LPCTSTR data, LPCTSTR regexp)
{
	CAtlRegExp<> regex;
	REParseError status = regex.Parse(regexp, FALSE);

	if (REPARSE_ERROR_OK != status) 
	{
		return FALSE;	
	}
	else
	{
		CAtlREMatchContext<> mc;
		if (!regex.Match(data, &mc))
		{
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	return TRUE;
}
void TipXMLValidater::Fix(LPCTSTR key, LPCTSTR data, LPTSTR newData, int len)
{
	m_nInvalidNumber++;
	_tcscpy_s(newData, len, data);
}

};